@@ -2,7 +2,7 @@ use 5.006;
 use strict;
 use warnings;
 package CPAN::Meta::Validator;
-our $VERSION = '2.131560'; # VERSION
+our $VERSION = '2.133380'; # VERSION
 
 
 #--------------------------------------------------------------------------#
@@ -78,10 +78,10 @@ my %definitions = (
   '2' => {
     # REQUIRED
     'abstract'            => { mandatory => 1, value => \&string  },
-    'author'              => { mandatory => 1, lazylist => { value => \&string } },
+    'author'              => { mandatory => 1, list => { value => \&string } },
     'dynamic_config'      => { mandatory => 1, value => \&boolean },
     'generated_by'        => { mandatory => 1, value => \&string  },
-    'license'             => { mandatory => 1, lazylist => { value => \&license } },
+    'license'             => { mandatory => 1, list => { value => \&license } },
     'meta-spec' => {
       mandatory => 1,
       'map' => {
@@ -96,7 +96,7 @@ my %definitions = (
 
     # OPTIONAL
     'description' => { value => \&string },
-    'keywords'    => { lazylist => { value => \&string } },
+    'keywords'    => { list => { value => \&string } },
     'no_index'    => $no_index_2,
     'optional_features'   => {
       'map'       => {
@@ -125,7 +125,7 @@ my %definitions = (
     },
     'resources'   => {
       'map'       => {
-        license    => { lazylist => { value => \&url } },
+        license    => { list => { value => \&url } },
         homepage   => { value => \&url },
         bugtracker => {
           'map' => {
@@ -426,7 +426,7 @@ sub new {
   # create an attributes hash
   my $self = {
     'data'    => $data,
-    'spec'    => $data->{'meta-spec'}{'version'} || "1.0",
+    'spec'    => eval { $data->{'meta-spec'}{'version'} } || "1.0",
     'errors'  => undef,
   };
 
@@ -452,7 +452,7 @@ sub errors {
 
 
 my $spec_error = "Missing validation action in specification. "
-  . "Must be one of 'map', 'list', 'lazylist', or 'value'";
+  . "Must be one of 'map', 'list', or 'value'";
 
 sub check_map {
     my ($self,$spec,$data) = @_;
@@ -484,8 +484,6 @@ sub check_map {
                 $self->check_map($spec->{$key}{'map'},$data->{$key});
             } elsif($spec->{$key}{'list'}) {
                 $self->check_list($spec->{$key}{'list'},$data->{$key});
-            } elsif($spec->{$key}{'lazylist'}) {
-                $self->check_lazylist($spec->{$key}{'lazylist'},$data->{$key});
             } else {
                 $self->_error( "$spec_error for '$key'" );
             }
@@ -498,8 +496,6 @@ sub check_map {
                 $self->check_map($spec->{':key'}{'map'},$data->{$key});
             } elsif($spec->{':key'}{'list'}) {
                 $self->check_list($spec->{':key'}{'list'},$data->{$key});
-            } elsif($spec->{':key'}{'lazylist'}) {
-                $self->check_lazylist($spec->{':key'}{'lazylist'},$data->{$key});
             } else {
                 $self->_error( "$spec_error for ':key'" );
             }
@@ -512,17 +508,6 @@ sub check_map {
     }
 }
 
-# if it's a string, make it into a list and check the list
-sub check_lazylist {
-    my ($self,$spec,$data) = @_;
-
-    if ( defined $data && ! ref($data) ) {
-      $data = [ $data ];
-    }
-
-    $self->check_list($spec,$data);
-}
-
 sub check_list {
     my ($self,$spec,$data) = @_;
 
@@ -545,8 +530,6 @@ sub check_list {
             $self->check_map($spec->{'map'},$value);
         } elsif(defined $spec->{'list'}) {
             $self->check_list($spec->{'list'},$value);
-        } elsif(defined $spec->{'lazylist'}) {
-            $self->check_lazylist($spec->{'lazylist'},$value);
         } elsif ($spec->{':key'}) {
             $self->check_map($spec,$value);
         } else {
@@ -832,7 +815,7 @@ __END__
 
 =pod
 
-=encoding utf-8
+=encoding UTF-8
 
 =head1 NAME
 
@@ -840,7 +823,7 @@ CPAN::Meta::Validator - validate CPAN distribution metadata structures
 
 =head1 VERSION
 
-version 2.131560
+version 2.133380
 
 =head1 SYNOPSIS
 
@@ -904,10 +887,6 @@ the appropriate specification definition.
 
 =item *
 
-check_lazylist($spec,$data)
-
-Checks whether a list conforms, but converts strings to a single-element list
-
 =back
 
 =head2 Validator Methods
@@ -1016,7 +995,7 @@ Validates that a given key is in an acceptable module name format, e.g.
 
 =end :internals
 
-=for Pod::Coverage anything boolean check_lazylist check_list custom_1 custom_2 exversion file
+=for Pod::Coverage anything boolean check_list custom_1 custom_2 exversion file
 identifier license module phase relation release_status string string_or_undef
 url urlspec version header check_map
 
@@ -1043,68 +1022,6 @@ Ricardo Signes <rjbs@cpan.org>
 
 =back
 
-=head1 CONTRIBUTORS
-
-=over 4
-
-=item *
-
-Ansgar Burchardt <ansgar@cpan.org>
-
-=item *
-
-Avar Arnfjord Bjarmason <avar@cpan.org>
-
-=item *
-
-Christopher J. Madsen <cjm@cpan.org>
-
-=item *
-
-Cory G Watson <gphat@cpan.org>
-
-=item *
-
-Damyan Ivanov <dam@cpan.org>
-
-=item *
-
-Eric Wilhelm <ewilhelm@cpan.org>
-
-=item *
-
-Gregor Hermann <gregoa@debian.org>
-
-=item *
-
-Ken Williams <kwilliams@cpan.org>
-
-=item *
-
-Kenichi Ishigaki <ishigaki@cpan.org>
-
-=item *
-
-Lars Dieckow <daxim@cpan.org>
-
-=item *
-
-Leon Timmermans <leont@cpan.org>
-
-=item *
-
-Mark Fowler <markf@cpan.org>
-
-=item *
-
-Michael G. Schwern <mschwern@cpan.org>
-
-=item *
-
-Randy Sims <randys@thepierianspring.org>
-
-=back
-
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2010 by David Golden and Ricardo Signes.
