@@ -8,7 +8,7 @@ use warnings;
 use Config;
 use Exporter;
 use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS);
-$VERSION     = "0.18";
+$VERSION     = "0.19";
 @ISA         = ("Exporter");
 @EXPORT_OK   = qw( plv2hash summary myconfig signature );
 %EXPORT_TAGS = (
@@ -97,8 +97,6 @@ my %BTD = map { $_ => 0 } qw(
     USE_ITHREADS
     USE_LARGE_FILES
     USE_LOCALE_COLLATE
-    USE_LOCALE_MESSAGES
-    USE_LOCALE_MONETARY
     USE_LOCALE_NUMERIC
     USE_LONG_DOUBLE
     USE_PERLIO
