@@ -1,10 +1,10 @@
 #
-# $Id: Encoder.pm,v 2.2 2011/08/09 07:49:44 dankogai Exp $
+# $Id: Encoder.pm,v 2.3 2013/09/14 07:51:59 dankogai Exp dankogai $
 #
 package Encode::Encoder;
 use strict;
 use warnings;
-our $VERSION = do { my @r = ( q$Revision: 2.2 $ =~ /\d+/g ); sprintf "%d." . "%02d" x $#r, @r };
+our $VERSION = do { my @r = ( q$Revision: 2.3 $ =~ /\d+/g ); sprintf "%d." . "%02d" x $#r, @r };
 
 require Exporter;
 our @ISA       = qw(Exporter);
@@ -211,7 +211,7 @@ To make the Base64 transcoder example above really work, you could
 write a module like this:
 
   package Encode::Base64;
-  use base 'Encode::Encoding';
+  use parent 'Encode::Encoding';
   __PACKAGE__->Define('base64');
   use MIME::Base64;
   sub encode{ 
