@@ -1,7 +1,7 @@
 package ExtUtils::MM_Any;
 
 use strict;
-our $VERSION = '6.68';
+our $VERSION = '6.84';
 
 use Carp;
 use File::Spec;
@@ -66,7 +66,7 @@ These are methods which help writing cross-platform code.
     my @os_flavor = $mm->os_flavor;
 
 @os_flavor is the style of operating system this is, usually
-corresponding to the MM_*.pm file we're using.  
+corresponding to the MM_*.pm file we're using.
 
 The first element of @os_flavor is the major family (ie. Unix,
 Windows, VMS, OS/2, etc...) and the rest are sub families.
@@ -80,7 +80,7 @@ Some examples:
     MacOS X        ('Unix',  'Darwin', 'MacOS', 'MacOS X')
     OS/2           ('OS/2')
 
-This is used to write code for styles of operating system.  
+This is used to write code for styles of operating system.
 See os_flavor_is() for use.
 
 
@@ -236,7 +236,7 @@ sub echo {
     $opts->{allow_variables} = 0 unless defined $opts->{allow_variables};
 
     my $ql_opts = { allow_variables => $opts->{allow_variables} };
-    my @cmds = map { '$(NOECHO) $(ECHO) '.$self->quote_literal($_, $ql_opts) } 
+    my @cmds = map { '$(NOECHO) $(ECHO) '.$self->quote_literal($_, $ql_opts) }
                split /\n/, $text;
     if( $file ) {
         my $redirect = $opts->{append} ? '>>' : '>';
@@ -519,7 +519,7 @@ sub clean {
 clean :: clean_subdirs
 ');
 
-    my @files = values %{$self->{XS}}; # .c files from *.xs files
+    my @files = sort values %{$self->{XS}}; # .c files from *.xs files
     my @dirs  = qw(blib);
 
     # Normally these are all under blib but they might have been
@@ -532,7 +532,7 @@ clean :: clean_subdirs
 #                    $(INST_BIN) $(INST_SCRIPT)
 #                    $(INST_MAN1DIR) $(INST_MAN3DIR)
 #                    $(INST_LIBDIR) $(INST_ARCHLIBDIR) $(INST_AUTODIR)
-#                    $(INST_STATIC) $(INST_DYNAMIC) $(INST_BOOT)
+#                    $(INST_STATIC) $(INST_DYNAMIC)
 #                 );
 
 
@@ -570,14 +570,15 @@ clean :: clean_subdirs
     push @dirs, $self->extra_clean_files;
 
     # Occasionally files are repeated several times from different sources
-    { my(%f) = map { ($_ => 1) } @files; @files = keys %f; }
-    { my(%d) = map { ($_ => 1) } @dirs;  @dirs  = keys %d; }
+    { my(%f) = map { ($_ => 1) } @files; @files = sort keys %f; }
+    { my(%d) = map { ($_ => 1) } @dirs;  @dirs  = sort keys %d; }
 
     push @m, map "\t$_\n", $self->split_command('- $(RM_F)',  @files);
     push @m, map "\t$_\n", $self->split_command('- $(RM_RF)', @dirs);
 
     # Leave Makefile.old around for realclean
     push @m, <<'MAKE';
+	  $(NOECHO) $(RM_F) $(MAKEFILE_OLD)
 	- $(MV) $(FIRST_MAKEFILE) $(MAKEFILE_OLD) $(DEV_NULL)
 MAKE
 
@@ -610,7 +611,7 @@ NOOP_FRAG
 
     for my $dir (@{$self->{DIR}}) {
         my $subclean = $self->oneliner(sprintf <<'CODE', $dir);
-chdir '%s';  system '$(MAKE) clean' if -f '$(FIRST_MAKEFILE)';
+exit 0 unless chdir '%s';  system '$(MAKE) clean' if -f '$(FIRST_MAKEFILE)';
 CODE
 
         $clean .= "\t$subclean\n";
@@ -726,7 +727,7 @@ sub dynamic {
 
     my($self) = shift;
     '
-dynamic :: $(FIRST_MAKEFILE) $(INST_DYNAMIC) $(INST_BOOT)
+dynamic :: $(FIRST_MAKEFILE) $(BOOTSTRAP) $(INST_DYNAMIC)
 	$(NOECHO) $(NOOP)
 ';
 }
@@ -769,7 +770,7 @@ sub manifypods_target {
     my $dependencies  = '';
 
     # populate manXpods & dependencies:
-    foreach my $name (keys %{$self->{MAN1PODS}}, keys %{$self->{MAN3PODS}}) {
+    foreach my $name (sort keys %{$self->{MAN1PODS}}, sort keys %{$self->{MAN3PODS}}) {
         $dependencies .= " \\\n\t$name";
     }
 
@@ -780,7 +781,7 @@ END
     my @man_cmds;
     foreach my $section (qw(1 3)) {
         my $pods = $self->{"MAN${section}PODS"};
-        push @man_cmds, $self->split_command(<<CMD, %$pods);
+        push @man_cmds, $self->split_command(<<CMD, map {($_,$pods->{$_})} sort keys %$pods);
 	\$(NOECHO) \$(POD2MAN) --section=$section --perm_rw=\$(PERM_RW)
 CMD
     }
@@ -822,13 +823,13 @@ MAKE_FRAG
         $self->{META_ADD}   || {},
         $self->{META_MERGE} || {},
     );
-    
+
     _fix_metadata_before_conversion( \%metadata );
 
     # paper over validation issues, but still complain, necessary because
     # there's no guarantee that the above will fix ALL errors
     my $meta = eval { CPAN::Meta->create( \%metadata, { lazy_validation => 1 } ) };
-    warn $@ if $@ and 
+    warn $@ if $@ and
                $@ !~ /encountered CODE.*, but JSON can only represent references to arrays or hashes/;
 
     # use the original metadata straight if the conversion failed
@@ -925,7 +926,7 @@ sub _fix_metadata_before_conversion {
 
     my @pairs = _sort_pairs($sort_sub, \%hash);
 
-Sorts the pairs of a hash based on keys ordered according 
+Sorts the pairs of a hash based on keys ordered according
 to C<$sort_sub>.
 
 =end private
@@ -986,7 +987,7 @@ sub metafile_data {
 
         generated_by => "ExtUtils::MakeMaker version $ExtUtils::MakeMaker::VERSION",
         'meta-spec'  => {
-            url         => 'http://module-build.sourceforge.net/META-spec-v1.4.html', 
+            url         => 'http://module-build.sourceforge.net/META-spec-v1.4.html',
             version     => 1.4
         },
     );
@@ -1086,7 +1087,7 @@ sub _add_requirements_to_meta_v2 {
     }
 
     $meta{prereqs}{runtime}{requires} = _normalize_prereqs($self->{PREREQ_PM})
-        if defined $self->{PREREQ_PM};
+        if $self->{ARGS}{PREREQ_PM};
     $meta{prereqs}{runtime}{requires}{perl} = _normalize_version($self->{MIN_PERL_VERSION})
         if $self->{MIN_PERL_VERSION};
 
@@ -1126,24 +1127,24 @@ sub _normalize_version {
 
 Implements a fake YAML dumper for a hash given
 as a list of pairs. No quoting/escaping is done. Keys
-are supposed to be strings. Values are undef, strings, 
+are supposed to be strings. Values are undef, strings,
 hash refs or array refs of strings.
 
 Supported options are:
 
     delta => STR - indentation delta
     use_header => BOOL - whether to include a YAML header
-    indent => STR - a string of spaces 
+    indent => STR - a string of spaces
           default: ''
 
     max_key_length => INT - maximum key length used to align
         keys and values of the same hash
         default: 20
-    key_sort => CODE - a sort sub 
+    key_sort => CODE - a sort sub
             It may be undef, which means no sorting by keys
         default: sub { lc $a cmp lc $b }
 
-    customs => HASH - special options for certain keys 
+    customs => HASH - special options for certain keys
            (whose values are hashes themselves)
         may contain: max_key_length, key_sort, customs
 
@@ -1159,8 +1160,8 @@ sub _dump_hash {
     # Use a list to preserve order.
     my @pairs;
 
-    my $k_sort 
-        = exists $options->{key_sort} ? $options->{key_sort} 
+    my $k_sort
+        = exists $options->{key_sort} ? $options->{key_sort}
                                       : sub { lc $a cmp lc $b };
     if ($k_sort) {
         croak "'key_sort' should be a coderef" unless ref $k_sort eq 'CODE';
@@ -1196,7 +1197,7 @@ sub _dump_hash {
                         $k_options{$k} = $k_custom{$k} if exists $k_custom{$k};
                     }
                 }
-                $yaml .= $indent . "$key:\n" 
+                $yaml .= $indent . "$key:\n"
                   . _dump_hash(\%k_options, %$val);
             }
             else {
@@ -1256,8 +1257,8 @@ sub metafile_file {
     my $self = shift;
 
     my %dump_options = (
-        use_header => 1, 
-        delta      => ' ' x 4, 
+        use_header => 1,
+        delta      => ' ' x 4,
         key_sort   => undef,
     );
     return _dump_hash(\%dump_options, @_);
@@ -1388,7 +1389,7 @@ sub write_mymeta {
     return unless _has_cpan_meta();
 
     _fix_metadata_before_conversion( $mymeta );
-    
+
     # this can still blow up
     # not sure if i should just eval this and skip file creation if it
     # blows up
@@ -1434,9 +1435,9 @@ sub realclean {
     { my(%f) = map { ($_ => 1) } @files;  @files = keys %f; }
     { my(%d) = map { ($_ => 1) } @dirs;   @dirs  = keys %d; }
 
-    my $rm_cmd  = join "\n\t", map { "$_" } 
+    my $rm_cmd  = join "\n\t", map { "$_" }
                     $self->split_command('- $(RM_F)',  @files);
-    my $rmf_cmd = join "\n\t", map { "$_" } 
+    my $rmf_cmd = join "\n\t", map { "$_" }
                     $self->split_command('- $(RM_RF)', @dirs);
 
     my $m = sprintf <<'MAKE', $rm_cmd, $rmf_cmd;
@@ -1522,7 +1523,7 @@ sub distsignature_target {
     my $self = shift;
 
     my $add_sign = $self->oneliner(<<'CODE', ['-MExtUtils::Manifest=maniadd']);
-eval { maniadd({q{SIGNATURE} => q{Public-key signature (added by MakeMaker)}}) } 
+eval { maniadd({q{SIGNATURE} => q{Public-key signature (added by MakeMaker)}}) }
     or print "Could not add SIGNATURE to MANIFEST: $${'@'}\n"
 CODE
 
@@ -1597,6 +1598,13 @@ sub init_ABSTRACT {
             carp "WARNING: Setting ABSTRACT via file ".
                  "'$self->{ABSTRACT_FROM}' failed\n";
     }
+
+    if ($self->{ABSTRACT} && $self->{ABSTRACT} =~ m![[:cntrl:]]+!) {
+            warn "WARNING: ABSTRACT contains control character(s),".
+                 " they will be removed\n";
+            $self->{ABSTRACT} =~ s![[:cntrl:]]+!!g;
+            return;
+    }
 }
 
 =head3 init_INST
@@ -1620,7 +1628,7 @@ sub init_INST {
     unless ($self->{INST_LIB}){
         if ($self->{PERL_CORE}) {
             if (defined $Cross::platform) {
-                $self->{INST_LIB} = $self->{INST_ARCHLIB} = 
+                $self->{INST_LIB} = $self->{INST_ARCHLIB} =
                   $self->catdir($self->{PERL_LIB},"..","xlib",
                                      $Cross::platform);
             }
@@ -1635,7 +1643,7 @@ sub init_INST {
     my @parentdir = split(/::/, $self->{PARENT_NAME});
     $self->{INST_LIBDIR}      = $self->catdir('$(INST_LIB)',     @parentdir);
     $self->{INST_ARCHLIBDIR}  = $self->catdir('$(INST_ARCHLIB)', @parentdir);
-    $self->{INST_AUTODIR}     = $self->catdir('$(INST_LIB)', 'auto', 
+    $self->{INST_AUTODIR}     = $self->catdir('$(INST_LIB)', 'auto',
                                               '$(FULLEXT)');
     $self->{INST_ARCHAUTODIR} = $self->catdir('$(INST_ARCHLIB)', 'auto',
                                               '$(FULLEXT)');
@@ -1685,7 +1693,7 @@ sub init_INSTALL_from_PREFIX {
 
     $self->init_lib2arch;
 
-    # There are often no Config.pm defaults for these new man variables so 
+    # There are often no Config.pm defaults for these new man variables so
     # we fall back to the old behavior which is to use installman*dir
     foreach my $num (1, 3) {
         my $k = 'installsiteman'.$num.'dir';
@@ -1710,7 +1718,7 @@ sub init_INSTALL_from_PREFIX {
       unless $Config{installsitescript};
 
     unless( $Config{installvendorbin} ) {
-        $self->{INSTALLVENDORBIN} ||= $Config{usevendorprefix} 
+        $self->{INSTALLVENDORBIN} ||= $Config{usevendorprefix}
                                     ? $Config{installbin}
                                     : '';
     }
@@ -1721,7 +1729,7 @@ sub init_INSTALL_from_PREFIX {
     }
 
 
-    my $iprefix = $Config{installprefixexp} || $Config{installprefix} || 
+    my $iprefix = $Config{installprefixexp} || $Config{installprefix} ||
                   $Config{prefixexp}        || $Config{prefix} || '';
     my $vprefix = $Config{usevendorprefix}  ? $Config{vendorprefixexp} : '';
     my $sprefix = $Config{siteprefixexp}    || '';
@@ -1765,7 +1773,7 @@ sub init_INSTALL_from_PREFIX {
           unless $Config{'installman'.$num.'dir'};
     }
 
-    my %bin_layouts = 
+    my %bin_layouts =
     (
         bin         => { s => $iprefix,
                          t => 'perl',
@@ -1786,7 +1794,7 @@ sub init_INSTALL_from_PREFIX {
                          t => 'site',
                          d => 'bin' },
     );
-    
+
     my %man_layouts =
     (
         man1dir         => { s => $iprefix,
@@ -1830,7 +1838,7 @@ sub init_INSTALL_from_PREFIX {
                          t => 'site',
                          d => 'site_perl',
                          style => $libstyle, },
-        
+
         archlib     => { s => $iprefix,
                          t => 'perl',
                          d => "$version/$arch",
@@ -1852,7 +1860,7 @@ sub init_INSTALL_from_PREFIX {
             my $Installvar = uc "install$var";
 
             if( $var =~ /arch/ ) {
-                $self->{$Installvar} ||= 
+                $self->{$Installvar} ||=
                   $self->catdir($self->{LIB}, $Config{archname});
             }
             else {
@@ -1880,7 +1888,7 @@ sub init_INSTALL_from_PREFIX {
         $d = "$style/$d" if $style;
         $self->prefixify($installvar, $s, $r, $d);
 
-        warn "  $Installvar == $self->{$Installvar}\n" 
+        warn "  $Installvar == $self->{$Installvar}\n"
           if $Verbose >= 2;
     }
 
@@ -1910,7 +1918,7 @@ $map{script} = $map{bin};
 sub init_INSTALL_from_INSTALL_BASE {
     my $self = shift;
 
-    @{$self}{qw(PREFIX VENDORPREFIX SITEPREFIX PERLPREFIX)} = 
+    @{$self}{qw(PREFIX VENDORPREFIX SITEPREFIX PERLPREFIX)} =
                                                          '$(INSTALL_BASE)';
 
     my %install;
@@ -1919,7 +1927,7 @@ sub init_INSTALL_from_INSTALL_BASE {
             my $uc_thing = uc $thing;
             my $key = "INSTALL".$dir.$uc_thing;
 
-            $install{$key} ||= 
+            $install{$key} ||=
               $self->catdir('$(INSTALL_BASE)', @{$map{$thing}});
         }
     }
@@ -1946,7 +1954,7 @@ MAKEMAKER: path to the MakeMaker module.
 
 MM_VERSION: ExtUtils::MakeMaker Version
 
-MM_REVISION: ExtUtils::MakeMaker version control revision (for backwards 
+MM_REVISION: ExtUtils::MakeMaker version control revision (for backwards
              compat)
 
 VERSION: version of your module
@@ -1983,8 +1991,12 @@ sub init_VERSION {
         }
     }
 
-    # strip blanks
     if (defined $self->{VERSION}) {
+        if ( $self->{VERSION} !~ /^\s*v?[\d_\.]+\s*$/ ) {
+          require version;
+          my $normal = eval { version->parse( $self->{VERSION} ) };
+          $self->{VERSION} = $normal if defined $normal;
+        }
         $self->{VERSION} =~ s/^\s+//;
         $self->{VERSION} =~ s/\s+$//;
     }
@@ -2029,12 +2041,14 @@ Defines at least these macros.
   SHELL             Program used to run shell commands
 
   ECHO              Print text adding a newline on the end
-  RM_F              Remove a file 
-  RM_RF             Remove a directory          
-  TOUCH             Update a file's timestamp   
-  TEST_F            Test for a file's existence 
-  CP                Copy a file                 
-  MV                Move a file                 
+  RM_F              Remove a file
+  RM_RF             Remove a directory
+  TOUCH             Update a file's timestamp
+  TEST_F            Test for a file's existence
+  TEST_S            Test the size of a file
+  CP                Copy a file
+  CP_NONEMPTY       Copy a file if it is not empty
+  MV                Move a file
   CHMOD             Change permissions on a file
   FALSE             Exit with non-zero
   TRUE              Exit with zero
@@ -2055,6 +2069,8 @@ sub init_tools {
     $self->{RM_F}     ||= $self->oneliner('rm_f',  ["-MExtUtils::Command"]);
     $self->{RM_RF}    ||= $self->oneliner('rm_rf', ["-MExtUtils::Command"]);
     $self->{TEST_F}   ||= $self->oneliner('test_f', ["-MExtUtils::Command"]);
+    $self->{TEST_S}   ||= $self->oneliner('test_s', ["-MExtUtils::Command::MM"]);
+    $self->{CP_NONEMPTY} ||= $self->oneliner('cp_nonempty', ["-MExtUtils::Command::MM"]);
     $self->{FALSE}    ||= $self->oneliner('exit 1');
     $self->{TRUE}     ||= $self->oneliner('exit 0');
 
@@ -2063,13 +2079,13 @@ sub init_tools {
     $self->{CP}       ||= $self->oneliner('cp', ["-MExtUtils::Command"]);
     $self->{MV}       ||= $self->oneliner('mv', ["-MExtUtils::Command"]);
 
-    $self->{MOD_INSTALL} ||= 
+    $self->{MOD_INSTALL} ||=
       $self->oneliner(<<'CODE', ['-MExtUtils::Install']);
 install([ from_to => {@ARGV}, verbose => '$(VERBINST)', uninstall_shadows => '$(UNINST)', dir_mode => '$(PERM_DIR)' ]);
 CODE
     $self->{DOC_INSTALL} ||= $self->oneliner('perllocal_install', ["-MExtUtils::Command::MM"]);
     $self->{UNINSTALL}   ||= $self->oneliner('uninstall', ["-MExtUtils::Command::MM"]);
-    $self->{WARN_IF_OLD_PACKLIST} ||= 
+    $self->{WARN_IF_OLD_PACKLIST} ||=
       $self->oneliner('warn_if_old_packlist', ["-MExtUtils::Command::MM"]);
     $self->{FIXIN}       ||= $self->oneliner('MY->fixin(shift)', ["-MExtUtils::MY"]);
     $self->{EQUALIZE_TIMESTAMP} ||= $self->oneliner('eqtime', ["-MExtUtils::Command"]);
@@ -2097,7 +2113,7 @@ CODE
     # Not everybody uses -f to indicate "use this Makefile instead"
     $self->{USEMAKEFILE}        ||= '-f';
 
-    # Some makes require a wrapper around macros passed in on the command 
+    # Some makes require a wrapper around macros passed in on the command
     # line.
     $self->{MACROSTART}         ||= '';
     $self->{MACROEND}           ||= '';
@@ -2131,13 +2147,17 @@ sub init_others {
         my(@libs) = $self->extliblist($libs);
         if ($libs[0] or $libs[1] or $libs[2]){
             # LD_RUN_PATH now computed by ExtUtils::Liblist
-            ($self->{EXTRALIBS},  $self->{BSLOADLIBS}, 
+            ($self->{EXTRALIBS},  $self->{BSLOADLIBS},
              $self->{LDLOADLIBS}, $self->{LD_RUN_PATH}) = @libs;
             last;
         }
     }
 
     if ( $self->{OBJECT} ) {
+        $self->{OBJECT} = join(" ", @{$self->{OBJECT}}) if ref $self->{OBJECT};
+        $self->{OBJECT} =~ s!\.o(bj)?\b!\$(OBJ_EXT)!g;
+    } elsif ( $self->{MAGICXS} && @{$self->{O_FILES}||[]} ) {
+        $self->{OBJECT} = join(" ", @{$self->{O_FILES}});
         $self->{OBJECT} =~ s!\.o(bj)?\b!\$(OBJ_EXT)!g;
     } else {
         # init_dirscan should have found out, if we have C files
@@ -2169,7 +2189,7 @@ sub init_others {
 sub _fix_libs {
     my($self, $libs) = @_;
 
-    return !defined $libs       ? ['']          : 
+    return !defined $libs       ? ['']          :
            !ref $libs           ? [$libs]       :
            !defined $libs->[0]  ? ['']          :
                                   $libs         ;
@@ -2180,7 +2200,7 @@ sub _fix_libs {
 
     my $make_frag = $MM->tools_other;
 
-Returns a make fragment containing definitions for the macros init_others() 
+Returns a make fragment containing definitions for the macros init_others()
 initializes.
 
 =cut
@@ -2191,7 +2211,7 @@ sub tools_other {
 
     # We set PM_FILTER as late as possible so it can see all the earlier
     # on macro-order sensitive makes such as nmake.
-    for my $tool (qw{ SHELL CHMOD CP MV NOOP NOECHO RM_F RM_RF TEST_F TOUCH 
+    for my $tool (qw{ SHELL CHMOD CP MV NOOP NOECHO RM_F RM_RF TEST_F TOUCH
                       UMASK_NULL DEV_NULL MKPATH EQUALIZE_TIMESTAMP
                       FALSE TRUE
                       ECHO ECHO_N
@@ -2202,7 +2222,8 @@ sub tools_other {
                       USEMAKEFILE
                       PM_FILTER
                       FIXIN
-                    } ) 
+                      CP_NONEMPTY
+                    } )
     {
         next unless defined $self->{$tool};
         push @m, "$tool = $self->{$tool}\n";
@@ -2354,8 +2375,8 @@ Used on the t/*.t files.
 sub test_via_harness {
     my($self, $perl, $tests) = @_;
 
-    return qq{\t$perl "-MExtUtils::Command::MM" }.
-           qq{"-e" "test_harness(\$(TEST_VERBOSE), '\$(INST_LIB)', '\$(INST_ARCHLIB)')" $tests\n};
+    return qq{\t$perl "-MExtUtils::Command::MM" "-MTest::Harness" }.
+           qq{"-e" "undef *Test::Harness::Switches; test_harness(\$(TEST_VERBOSE), '\$(INST_LIB)', '\$(INST_ARCHLIB)')" $tests\n};
 }
 
 =head3 test_via_script
@@ -2387,7 +2408,7 @@ pm_to_blib soon.
 sub tool_autosplit {
     my($self, %attribs) = @_;
 
-    my $maxlen = $attribs{MAXLEN} ? '$$AutoSplit::Maxlen=$attribs{MAXLEN};' 
+    my $maxlen = $attribs{MAXLEN} ? '$$AutoSplit::Maxlen=$attribs{MAXLEN};'
                                   : '';
 
     my $asplit = $self->oneliner(sprintf <<'PERL_CODE', $maxlen);
@@ -2444,11 +2465,11 @@ sub arch_check {
         my $arch = (grep length, $self->splitdir($pthinks))[-1];
 
         print <<END unless $self->{UNINSTALLED_PERL};
-Your perl and your Config.pm seem to have different ideas about the 
+Your perl and your Config.pm seem to have different ideas about the
 architecture they are running on.
 Perl thinks: [$arch]
 Config says: [$Config{archname}]
-This may or may not cause problems. Please check your installation of perl 
+This may or may not cause problems. Please check your installation of perl
 if you have problems building this extension.
 END
     }
@@ -2498,6 +2519,44 @@ sub find_tests {
     return -d 't' ? 't/*.t' : '';
 }
 
+=head3 find_tests_recursive
+
+  my $tests = $mm->find_tests_recursive;
+
+Returns a string suitable for feeding to the shell to return all
+tests in t/ but recursively.
+
+=cut
+
+sub find_tests_recursive {
+    my($self) = shift;
+    return '' unless -d 't';
+
+    require File::Find;
+
+    my %testfiles;
+
+    my $wanted = sub {
+        return unless m!\.t$!;
+        my ($volume,$directories,$file) =
+            File::Spec->splitpath( $File::Find::name  );
+        my @dirs = File::Spec->splitdir( $directories );
+        for ( @dirs ) {
+          next if $_ eq 't';
+          unless ( $_ ) {
+            $_ = '*.t';
+            next;
+          }
+          $_ = '*';
+        }
+        my $testfile = join '/', @dirs;
+        $testfiles{ $testfile } = 1;
+    };
+
+    File::Find::find( $wanted, 't' );
+
+    return join ' ', sort keys %testfiles;
+}
 
 =head3 extra_clean_files
 
@@ -2509,7 +2568,7 @@ addition to the usual set.
 =cut
 
 # An empty method here tickled a perl 5.8.1 bug and would return its object.
-sub extra_clean_files { 
+sub extra_clean_files {
     return;
 }
 
@@ -2550,7 +2609,7 @@ installation.
 sub libscan {
     my($self,$path) = @_;
     my($dirs,$file) = ($self->splitpath($path))[1,2];
-    return '' if grep /^(?:RCS|CVS|SCCS|\.svn|_darcs)$/, 
+    return '' if grep /^(?:RCS|CVS|SCCS|\.svn|_darcs)$/,
                      $self->splitdir($dirs), $file;
 
     return $path;
