@@ -5,7 +5,7 @@ use strict;
 use ExtUtils::MakeMaker qw(neatvalue);
 use File::Spec;
 
-our $VERSION = '6.64';
+our $VERSION = '6.84';
 
 require ExtUtils::MM_Any;
 require ExtUtils::MM_Unix;
@@ -81,12 +81,12 @@ $self->{BASEEXT}.def: Makefile.PL
 	}
 	close $imp or die "Can't close tmpimp.imp";
 	# print "emximp -o tmpimp$Config::Config{lib_ext} tmpimp.imp\n";
-	system "emximp -o tmpimp$Config::Config{lib_ext} tmpimp.imp" 
+	system "emximp -o tmpimp$Config::Config{lib_ext} tmpimp.imp"
 	    and die "Cannot make import library: $!, \$?=$?";
 	# May be running under miniperl, so have no glob...
-	eval "unlink <tmp_imp/*>; 1" or system "rm tmp_imp/*";
-	system "cd tmp_imp; $Config::Config{ar} x ../tmpimp$Config::Config{lib_ext}" 
-	    and die "Cannot extract import objects: $!, \$?=$?";      
+	eval { unlink <tmp_imp/*>; 1 } or system "rm tmp_imp/*";
+	system "cd tmp_imp; $Config::Config{ar} x ../tmpimp$Config::Config{lib_ext}"
+	    and die "Cannot extract import objects: $!, \$?=$?";
     }
     join('',@m);
 }
@@ -95,7 +95,7 @@ sub static_lib {
     my($self) = @_;
     my $old = $self->ExtUtils::MM_Unix::static_lib();
     return $old unless $self->{IMPORTS} && %{$self->{IMPORTS}};
-    
+
     my @chunks = split /\n{2,}/, $old;
     shift @chunks unless length $chunks[0]; # Empty lines at the start
     $chunks[0] .= <<'EOC';
