@@ -15,14 +15,14 @@ use ExtUtils::MakeMaker qw($Verbose neatvalue);
 
 # If we make $VERSION an our variable parse_version() breaks
 use vars qw($VERSION);
-$VERSION = '6.63_02';
-$VERSION = eval $VERSION;
+$VERSION = '6.84';
+$VERSION = eval $VERSION;  ## no critic [BuiltinFunctions::ProhibitStringyEval]
 
 require ExtUtils::MM_Any;
 our @ISA = qw(ExtUtils::MM_Any);
 
 my %Is;
-BEGIN { 
+BEGIN {
     $Is{OS2}     = $^O eq 'os2';
     $Is{Win32}   = $^O eq 'MSWin32' || $Config{osname} eq 'NetWare';
     $Is{Dos}     = $^O eq 'dos';
@@ -129,10 +129,10 @@ sub c_o {
     my($self) = shift;
     return '' unless $self->needs_linking();
     my(@m);
-    
+
     my $command = '$(CCCMD)';
     my $flags   = '$(CCCDLFLAGS) "-I$(PERL_INC)" $(PASTHRU_DEFINE) $(DEFINE)';
-    
+
     if (my $cpp = $Config{cpprun}) {
         my $cpp_cmd = $self->const_cccmd;
         $cpp_cmd =~ s/^CCCMD\s*=\s*\$\(CC\)/$cpp/;
@@ -217,7 +217,7 @@ sub cflags {
     ( $name = $self->{NAME} . "_cflags" ) =~ s/:/_/g ;
     if ($prog = $Config{$name}) {
 	# Expand hints for this extension via the shell
-	print STDOUT "Processing $name hint:\n" if $Verbose;
+	print "Processing $name hint:\n" if $Verbose;
 	my(@o)=`cc=\"$cflags{cc}\"
 	  ccflags=\"$cflags{ccflags}\"
 	  optimize=\"$cflags{optimize}\"
@@ -234,9 +234,9 @@ sub cflags {
 	    chomp $line;
 	    if ($line =~ /(.*?)=\s*(.*)\s*$/){
 		$cflags{$1} = $2;
-		print STDOUT "	$1 = $2\n" if $Verbose;
+		print "	$1 = $2\n" if $Verbose;
 	    } else {
-		print STDOUT "Unrecognised result from hint: '$line'\n";
+		print "Unrecognised result from hint: '$line'\n";
 	    }
 	}
     }
@@ -372,7 +372,7 @@ sub constants {
     for my $macro (qw(
 
               AR_STATIC_ARGS DIRFILESEP DFSEP
-              NAME NAME_SYM 
+              NAME NAME_SYM
               VERSION    VERSION_MACRO    VERSION_SYM DEFINE_VERSION
               XS_VERSION XS_VERSION_MACRO             XS_DEFINE_VERSION
               INST_ARCHLIB INST_SCRIPT INST_BIN INST_LIB
@@ -385,18 +385,18 @@ sub constants {
                           "DESTINSTALL".$_)
                         } $self->installvars),
                    qw(
-              PERL_LIB    
+              PERL_LIB
               PERL_ARCHLIB
               LIBPERL_A MYEXTLIB
-              FIRST_MAKEFILE MAKEFILE_OLD MAKE_APERL_FILE 
-              PERLMAINCC PERL_SRC PERL_INC 
+              FIRST_MAKEFILE MAKEFILE_OLD MAKE_APERL_FILE
+              PERLMAINCC PERL_SRC PERL_INC
               PERL            FULLPERL          ABSPERL
               PERLRUN         FULLPERLRUN       ABSPERLRUN
               PERLRUNINST     FULLPERLRUNINST   ABSPERLRUNINST
               PERL_CORE
               PERM_DIR PERM_RW PERM_RWX
 
-	      ) ) 
+	      ) )
     {
 	next unless defined $self->{$macro};
 
@@ -423,7 +423,7 @@ MM_REVISION = $self->{MM_REVISION}
               MAKE
 	      FULLEXT BASEEXT PARENT_NAME DLBASE VERSION_FROM INC DEFINE OBJECT
 	      LDFROM LINKTYPE BOOTDEP
-	      /	) 
+	      /	)
     {
 	next unless defined $self->{$macro};
 	push @m, "$macro = $self->{$macro}\n";
@@ -443,7 +443,7 @@ MAN3PODS = ".$self->wraplist(sort keys %{$self->{MAN3PODS}})."
     push @m, q{
 # Where is the Config information that we are using/depend on
 CONFIGDEP = $(PERL_ARCHLIB)$(DFSEP)Config.pm $(PERL_INC)$(DFSEP)config.h
-};
+} if -e File::Spec->catfile( $self->{PERL_INC}, 'config.h' );
 
 
     push @m, qq{
@@ -471,7 +471,7 @@ PERL_ARCHIVE_AFTER = $self->{PERL_ARCHIVE_AFTER}
 
 TO_INST_PM = ".$self->wraplist(sort keys %{$self->{PM}})."
 
-PM_TO_BLIB = ".$self->wraplist(%{$self->{PM}})."
+PM_TO_BLIB = ".$self->wraplist(map { ($_ => $self->{PM}->{$_}) } sort keys %{$self->{PM}})."
 ";
 
     join('',@m);
@@ -533,18 +533,18 @@ Defines a lot of macros for distribution support.
 
   COMPRESS      compression command to          gzip --best
                 use for tarfiles
-  SUFFIX        suffix to put on                .gz 
+  SUFFIX        suffix to put on                .gz
                 compressed files
 
   SHAR          shar command to use             shar
 
   PREOP         extra commands to run before
-                making the archive 
+                making the archive
   POSTOP        extra commands to run after
                 making the archive
 
   TO_UNIX       a command to convert linefeeds
-                to Unix style in your archive 
+                to Unix style in your archive
 
   CI            command to checkin your         ci -u
                 sources to version control
@@ -583,7 +583,6 @@ sub init_dist {
 
     ($self->{DISTNAME} = $self->{NAME}) =~ s{::}{-}g unless $self->{DISTNAME};
     $self->{DISTVNAME} ||= $self->{DISTNAME}.'-'.$self->{VERSION};
-
 }
 
 =item dist (o)
@@ -601,7 +600,10 @@ sub dist {
     my($self, %attribs) = @_;
 
     my $make = '';
-    foreach my $key (qw( 
+    if ( $attribs{SUFFIX} && $attribs{SUFFIX} !~ m!^\.! ) {
+      $attribs{SUFFIX} = '.' . $attribs{SUFFIX};
+    }
+    foreach my $key (qw(
             TAR TARFLAGS ZIP ZIPFLAGS COMPRESS SUFFIX SHAR
             PREOP POSTOP TO_UNIX
             CI RCS_LABEL DIST_CP DIST_DEFAULT
@@ -638,7 +640,7 @@ manifest :
 	$(PERLRUN) "-MExtUtils::Manifest=mkmanifest" -e mkmanifest
 
 veryclean : realclean
-	$(RM_F) *~ */*~ *.orig */*.orig *.bak */*.bak *.old */*.old 
+	$(RM_F) *~ */*~ *.orig */*.orig *.bak */*.bak *.old */*.old
 
 MAKE_FRAG
 
@@ -674,7 +676,7 @@ sub dist_core {
     my($self) = shift;
 
     my $make_frag = '';
-    foreach my $target (qw(dist tardist uutardist tarfile zipdist zipfile 
+    foreach my $target (qw(dist tardist uutardist tarfile zipdist zipfile
                            shdist))
     {
         my $method = $target.'_target';
@@ -768,6 +770,7 @@ $(DISTVNAME).tar$(SUFFIX) : distdir
 	$(TAR) $(TARFLAGS) $(DISTVNAME).tar $(DISTVNAME)
 	$(RM_RF) $(DISTVNAME)
 	$(COMPRESS) $(DISTVNAME).tar
+	$(NOECHO) $(ECHO) 'Created $(DISTVNAME).tar$(SUFFIX)'
 	$(POSTOP)
 MAKE_FRAG
 }
@@ -790,6 +793,7 @@ $(DISTVNAME).zip : distdir
 	$(PREOP)
 	$(ZIP) $(ZIPFLAGS) $(DISTVNAME).zip $(DISTVNAME)
 	$(RM_RF) $(DISTVNAME)
+	$(NOECHO) $(ECHO) 'Created $(DISTVNAME).zip'
 	$(POSTOP)
 MAKE_FRAG
 }
@@ -808,6 +812,7 @@ sub uutardist_target {
     return <<'MAKE_FRAG';
 uutardist : $(DISTVNAME).tar$(SUFFIX)
 	uuencode $(DISTVNAME).tar$(SUFFIX) $(DISTVNAME).tar$(SUFFIX) > $(DISTVNAME).tar$(SUFFIX)_uu
+	$(NOECHO) $(ECHO) 'Created $(DISTVNAME).tar$(SUFFIX)_uu'
 MAKE_FRAG
 }
 
@@ -828,6 +833,7 @@ shdist : distdir
 	$(PREOP)
 	$(SHAR) $(DISTVNAME) > $(DISTVNAME).shar
 	$(RM_RF) $(DISTVNAME)
+	$(NOECHO) $(ECHO) 'Created $(DISTVNAME).shar'
 	$(POSTOP)
 MAKE_FRAG
 }
@@ -873,11 +879,6 @@ $(BOOTSTRAP) : $(FIRST_MAKEFILE) $(BOOTDEP) $(INST_ARCHAUTODIR)$(DFSEP).exists
 		-e "Mkbootstrap('$(BASEEXT)','$(BSLOADLIBS)');"
 	$(NOECHO) $(TOUCH) %s
 	$(CHMOD) $(PERM_RW) %s
-
-$(INST_BOOT) : $(BOOTSTRAP) $(INST_ARCHAUTODIR)$(DFSEP).exists
-	$(NOECHO) $(RM_RF) %s
-	- $(CP) $(BOOTSTRAP) %s
-	$(CHMOD) $(PERM_RW) %s
 MAKE_FRAG
 }
 
@@ -909,7 +910,7 @@ OTHERLDFLAGS = '.$ld_opt.$otherldflags.'
 INST_DYNAMIC_DEP = '.$inst_dynamic_dep.'
 INST_DYNAMIC_FIX = '.$ld_fix.'
 
-$(INST_DYNAMIC): $(OBJECT) $(MYEXTLIB) $(BOOTSTRAP) $(INST_ARCHAUTODIR)$(DFSEP).exists $(EXPORT_LIST) $(PERL_ARCHIVE) $(PERL_ARCHIVE_AFTER) $(INST_DYNAMIC_DEP)
+$(INST_DYNAMIC): $(OBJECT) $(MYEXTLIB) $(INST_ARCHAUTODIR)$(DFSEP).exists $(EXPORT_LIST) $(PERL_ARCHIVE) $(PERL_ARCHIVE_AFTER) $(INST_DYNAMIC_DEP)
 ');
     if ($armaybe ne ':'){
 	$ldfrom = 'tmp$(LIB_EXT)';
@@ -919,7 +920,7 @@ $(INST_DYNAMIC): $(OBJECT) $(MYEXTLIB) $(BOOTSTRAP) $(INST_ARCHAUTODIR)$(DFSEP).
     $ldfrom = "-all $ldfrom -none" if $Is{OSF};
 
     # The IRIX linker doesn't use LD_RUN_PATH
-    my $ldrun = $Is{IRIX} && $self->{LD_RUN_PATH} ?         
+    my $ldrun = $Is{IRIX} && $self->{LD_RUN_PATH} ?
                        qq{-rpath "$self->{LD_RUN_PATH}"} : '';
 
     # For example in AIX the shared objects/libraries from previous builds
@@ -957,6 +958,8 @@ MAKE
 
     push @m, <<'MAKE';
 	$(CHMOD) $(PERM_RWX) $@
+	$(NOECHO) $(RM_RF) $(BOOTSTRAP)
+	- $(CP_NONEMPTY) $(BOOTSTRAP) $(INST_BOOT) $(PERM_RW)
 MAKE
 
     return join('',@m);
@@ -1025,7 +1028,7 @@ WARNING
             my ($abs, $val);
             if ($self->file_name_is_absolute($name)) {     # /foo/bar
                 $abs = $name;
-            } elsif ($self->canonpath($name) eq 
+            } elsif ($self->canonpath($name) eq
                      $self->canonpath(basename($name))) {  # foo
                 $abs = $self->catfile($dir, $name);
             } else {                                            # foo/bar
@@ -1064,7 +1067,7 @@ WARNING
             }
         }
     }
-    print STDOUT "Unable to find a perl $ver (by these names: @$names, in these dirs: @$dirs)\n";
+    print "Unable to find a perl $ver (by these names: @$names, in these dirs: @$dirs)\n";
     0; # false and not empty
 }
 
@@ -1087,7 +1090,7 @@ sub fixin {    # stolen from the pink Camel book, more or less
         open( my $fixin, '<', $file ) or croak "Can't process '$file': $!";
         local $/ = "\n";
         chomp( my $line = <$fixin> );
-        next unless $line =~ s/^\s*\#!\s*//;    # Not a shbang file.
+        next unless $line =~ s/^\s*\#!\s*//;    # Not a shebang file.
 
         my $shb = $self->_fixin_replace_shebang( $file, $line );
         next unless defined $shb;
@@ -1162,7 +1165,7 @@ sub _fixin_replace_shebang {
         my (@absdirs)
             = reverse grep { $self->file_name_is_absolute($_) } $self->path;
         $interpreter = '';
-     
+
          foreach my $dir (@absdirs) {
             if ( $self->maybe_command($cmd) ) {
                 warn "Ignoring $interpreter in $file\n"
@@ -1173,11 +1176,11 @@ sub _fixin_replace_shebang {
     }
 
     # Figure out how to invoke interpreter on this machine.
- 
+
     my ($does_shbang) = $Config{'sharpbang'} =~ /^\s*\#\!/;
     my ($shb) = "";
     if ($interpreter) {
-        print STDOUT "Changing sharpbang in $file to $interpreter"
+        print "Changing sharpbang in $file to $interpreter"
             if $Verbose;
          # this is probably value-free on DOSISH platforms
         if ($does_shbang) {
@@ -1193,7 +1196,7 @@ eval 'exec $interpreter $arg -S \$0 \${1+"\$\@"}'
     else {
         warn "Can't find $cmd in PATH, $file unchanged"
             if $Verbose;
-        return undef;
+        return;
     }
     return $shb
 }
@@ -1263,7 +1266,7 @@ Called by init_main.
 
 sub init_dirscan {	# --- File and Directory Lists (.xs .pm .pod etc)
     my($self) = @_;
-    my(%dir, %xs, %c, %h, %pl_files, %pm);
+    my(%dir, %xs, %c, %o, %h, %pl_files, %pm);
 
     my %ignore = map {( $_ => 1 )} qw(Makefile.PL Build.PL test.pl t);
 
@@ -1271,10 +1274,22 @@ sub init_dirscan {	# --- File and Directory Lists (.xs .pm .pod etc)
     $Is{VMS} ? $ignore{"$self->{DISTVNAME}.dir"} = 1
             : $ignore{$self->{DISTVNAME}} = 1;
 
+    my $distprefix = $Is{VMS} ? qr/^\Q$self->{DISTNAME}\E-.*\.dir$/i
+                              : qr/^\Q$self->{DISTNAME}-/;
+
     @ignore{map lc, keys %ignore} = values %ignore if $Is{VMS};
 
+    if ( defined $self->{XS} and !defined $self->{C} ) {
+	my @c_files = grep { m/\.c(pp|xx)?\z/i } values %{$self->{XS}};
+	my @o_files = grep { m/(?:.(?:o(?:bj)?)|\$\(OBJ_EXT\))\z/i } values %{$self->{XS}};
+	%c = map { $_ => 1 } @c_files;
+	%o = map { $_ => 1 } @o_files;
+    }
+
     foreach my $name ($self->lsdir($Curdir)){
 	next if $name =~ /\#/;
+	next if $name =~ $distprefix;
+	$name = lc($name) if $Is{VMS};
 	next if $name eq $Curdir or $name eq $Updir or $ignore{$name};
 	next unless $self->libscan($name);
 	if (-d $name){
@@ -1300,7 +1315,7 @@ sub init_dirscan {	# --- File and Directory Lists (.xs .pm .pod etc)
 		($pl_files{$name} = $name) =~ s/[._]pl\z//i ;
 	    }
 	    else {
-                $pm{$name} = $self->catfile($self->{INST_LIBDIR},$name); 
+                $pm{$name} = $self->catfile($self->{INST_LIBDIR},$name);
             }
 	} elsif ($name =~ /\.(p[ml]|pod)\z/){
 	    $pm{$name} = $self->catfile($self->{INST_LIBDIR},$name);
@@ -1315,7 +1330,8 @@ sub init_dirscan {	# --- File and Directory Lists (.xs .pm .pod etc)
     $self->{PM}         ||= \%pm;
 
     my @o_files = @{$self->{C}};
-    $self->{O_FILES} = [grep s/\.c(pp|xx|c)?\z/$self->{OBJ_EXT}/i, @o_files];
+    %o = (%o, map { $_ => 1 } grep s/\.c(pp|xx|c)?\z/$self->{OBJ_EXT}/i, @o_files);
+    $self->{O_FILES} = [sort keys %o];
 }
 
 
@@ -1379,7 +1395,7 @@ sub init_MAN1PODS {
 	    next unless $self->_has_pod($name);
 
 	    $self->{MAN1PODS}->{$name} =
-		$self->catfile("\$(INST_MAN1DIR)", 
+		$self->catfile("\$(INST_MAN1DIR)",
 			       basename($name).".\$(MAN1EXT)");
 	}
     }
@@ -1411,7 +1427,7 @@ sub init_MAN3PODS {
     my $parentlibs_re = join '|', @{$self->{PMLIBPARENTDIRS}};
 
     # Remove "Configure.pm" and similar, if it's not the only pod listed
-    # To force inclusion, just name it "Configure.pod", or override 
+    # To force inclusion, just name it "Configure.pod", or override
     # MAN3PODS
     foreach my $name (keys %manifypods) {
 	if ($self->{PERL_CORE} and $name =~ /(config|setup).*\.pm/is) {
@@ -1516,7 +1532,7 @@ sub init_PM {
             my $striplibpath;
 
 	    my $parentlibs_re = join '|', @{$self->{PMLIBPARENTDIRS}};
-	    $prefix =  $self->{INST_LIB} 
+	    $prefix =  $self->{INST_LIB}
                 if ($striplibpath = $path) =~ s{^(\W*)($parentlibs_re)\W}
 	                                       {$1}i;
 
@@ -1542,7 +1558,7 @@ sub init_DIRFILESEP {
 
     $self->{DIRFILESEP} = '/';
 }
-    
+
 
 =item init_main
 
@@ -1620,15 +1636,15 @@ sub init_main {
 	$self->{PERL_LIB}     ||= $self->catdir("$self->{PERL_SRC}","lib");
 
         if (defined $Cross::platform) {
-            $self->{PERL_ARCHLIB} = 
+            $self->{PERL_ARCHLIB} =
               $self->catdir("$self->{PERL_SRC}","xlib",$Cross::platform);
-            $self->{PERL_INC}     = 
-              $self->catdir("$self->{PERL_SRC}","xlib",$Cross::platform, 
+            $self->{PERL_INC}     =
+              $self->catdir("$self->{PERL_SRC}","xlib",$Cross::platform,
                                  $Is{Win32}?("CORE"):());
         }
         else {
             $self->{PERL_ARCHLIB} = $self->{PERL_LIB};
-            $self->{PERL_INC}     = ($Is{Win32}) ? 
+            $self->{PERL_INC}     = ($Is{Win32}) ?
               $self->catdir($self->{PERL_LIB},"CORE") : $self->{PERL_SRC};
         }
 
@@ -1682,12 +1698,12 @@ from the perl source tree.
 		$self->{PERL_ARCHLIB}	   = $lib;
 		$self->{PERL_INC}	   = $inc;
 		$self->{UNINSTALLED_PERL}  = 1;
-		print STDOUT <<EOP;
+		print <<EOP;
 ... Detected uninstalled Perl.  Trying to continue.
 EOP
 	      }
 	    }
-	}	
+	}
     }
 
     # We get SITELIBEXP and SITEARCHEXP directly via
@@ -1701,7 +1717,7 @@ EOP
     $self->{MAN3EXT} ||= $Config{man3ext};
 
     # Get some stuff out of %Config if we haven't yet done so
-    print STDOUT "CONFIG must be an array ref\n"
+    print "CONFIG must be an array ref\n"
         if ($self->{CONFIG} and ref $self->{CONFIG} ne 'ARRAY');
     $self->{CONFIG} = [] unless (ref $self->{CONFIG});
     push(@{$self->{CONFIG}}, @ExtUtils::MakeMaker::Get_from_Config);
@@ -1709,7 +1725,7 @@ EOP
     my(%once_only);
     foreach my $m (@{$self->{CONFIG}}){
         next if $once_only{$m};
-        print STDOUT "CONFIG key '$m' does not exist in Config.pm\n"
+        print "CONFIG key '$m' does not exist in Config.pm\n"
                 unless exists $Config{$m};
         $self->{uc $m} ||= $Config{$m};
         $once_only{$m} = 1;
@@ -1754,6 +1770,7 @@ sub init_tools {
     $self->{RM_RF}      ||= "rm -rf";
     $self->{TOUCH}      ||= "touch";
     $self->{TEST_F}     ||= "test -f";
+    $self->{TEST_S}     ||= "test -s";
     $self->{CP}         ||= "cp";
     $self->{MV}         ||= "mv";
     $self->{CHMOD}      ||= "chmod";
@@ -1805,7 +1822,7 @@ sub init_lib2arch {
     # architecture. If not we take it as a sign that it should be the
     # same as the requested installation directory. Otherwise we take
     # the found one.
-    for my $libpair ({l=>"privlib",   a=>"archlib"}, 
+    for my $libpair ({l=>"privlib",   a=>"archlib"},
                      {l=>"sitelib",   a=>"sitearch"},
                      {l=>"vendorlib", a=>"vendorarch"},
                     )
@@ -1819,11 +1836,11 @@ sub init_lib2arch {
             $self->prefixify($Arch,$ilib,$self->{$Lib});
 
             unless (-d $self->{$Arch}) {
-                print STDOUT "Directory $self->{$Arch} not found\n" 
+                print "Directory $self->{$Arch} not found\n"
                   if $Verbose;
                 $self->{$Arch} = $self->{$Lib};
             }
-            print STDOUT "Defaulting $Arch to $self->{$Arch}\n" if $Verbose;
+            print "Defaulting $Arch to $self->{$Arch}\n" if $Verbose;
         }
     }
 }
@@ -1853,15 +1870,15 @@ sub init_PERL {
     my($self) = shift;
 
     my @defpath = ();
-    foreach my $component ($self->{PERL_SRC}, $self->path(), 
-                           $Config{binexp}) 
+    foreach my $component ($self->{PERL_SRC}, $self->path(),
+                           $Config{binexp})
     {
 	push @defpath, $component if defined $component;
     }
 
     # Build up a set of file names (not command names).
     my $thisperl = $self->canonpath($^X);
-    $thisperl .= $Config{exe_ext} unless 
+    $thisperl .= $Config{exe_ext} unless
                 # VMS might have a file version # at the end
       $Is{VMS} ? $thisperl =~ m/$Config{exe_ext}(;\d+)?$/i
               : $thisperl =~ m/$Config{exe_ext}$/i;
@@ -1871,9 +1888,9 @@ sub init_PERL {
 
     my @perls = ($thisperl);
     push @perls, map { "$_$Config{exe_ext}" }
-                     ('perl', 'perl5', "perl$Config{version}");
+                     ("perl$Config{version}", 'perl5', 'perl');
 
-    # miniperl has priority over all but the cannonical perl when in the
+    # miniperl has priority over all but the canonical perl when in the
     # core.  Otherwise its a last resort.
     my $miniperl = "miniperl$Config{exe_ext}";
     if( $self->{PERL_CORE} ) {
@@ -1890,13 +1907,13 @@ sub init_PERL {
 
     # When built for debugging, VMS doesn't create perl.exe but ndbgperl.exe.
     my $perl_name = 'perl';
-    $perl_name = 'ndbgperl' if $Is{VMS} && 
+    $perl_name = 'ndbgperl' if $Is{VMS} &&
       defined $Config{usevmsdebug} && $Config{usevmsdebug} eq 'define';
 
     # XXX This logic is flawed.  If "miniperl" is anywhere in the path
     # it will get confused.  It should be fixed to work only on the filename.
     # Define 'FULLPERL' to be a non-miniperl (used in test: target)
-    ($self->{FULLPERL} = $self->{PERL}) =~ s/miniperl/$perl_name/i
+    ($self->{FULLPERL} = $self->{PERL}) =~ s/\Q$miniperl\E$/$perl_name$Config{exe_ext}/i
 	unless $self->{FULLPERL};
 
     # Little hack to get around VMS's find_perl putting "MCR" in front
@@ -1927,10 +1944,10 @@ sub init_PERL {
         $self->{$run}  = "\$($perl)";
 
         # Make sure perl can find itself before it's installed.
-        $self->{$run} .= q{ "-I$(PERL_LIB)" "-I$(PERL_ARCHLIB)"} 
+        $self->{$run} .= q{ "-I$(PERL_LIB)" "-I$(PERL_ARCHLIB)"}
           if $self->{UNINSTALLED_PERL} || $self->{PERL_CORE};
 
-        $self->{$perl.'RUNINST'} = 
+        $self->{$perl.'RUNINST'} =
           sprintf q{$(%sRUN) "-I$(INST_ARCHLIB)" "-I$(INST_LIB)"}, $perl;
     }
 
@@ -2002,18 +2019,18 @@ sub init_xs {
     my $self = shift;
 
     if ($self->has_link_code()) {
-        $self->{INST_STATIC}  = 
+        $self->{INST_STATIC}  =
           $self->catfile('$(INST_ARCHAUTODIR)', '$(BASEEXT)$(LIB_EXT)');
-        $self->{INST_DYNAMIC} = 
+        $self->{INST_DYNAMIC} =
           $self->catfile('$(INST_ARCHAUTODIR)', '$(DLBASE).$(DLEXT)');
-        $self->{INST_BOOT}    = 
+        $self->{INST_BOOT}    =
           $self->catfile('$(INST_ARCHAUTODIR)', '$(BASEEXT).bs');
     } else {
         $self->{INST_STATIC}  = '';
         $self->{INST_DYNAMIC} = '';
         $self->{INST_BOOT}    = '';
     }
-}    
+}
 
 =item install (o)
 
@@ -2052,9 +2069,15 @@ doc__install : doc_site_install
 
 pure_perl_install :: all
 	$(NOECHO) $(MOD_INSTALL) \
-		read }.$self->catfile('$(PERL_ARCHLIB)','auto','$(FULLEXT)','.packlist').q{ \
+};
+
+    push @m,
+q{		read }.$self->catfile('$(PERL_ARCHLIB)','auto','$(FULLEXT)','.packlist').q{ \
 		write }.$self->catfile('$(DESTINSTALLARCHLIB)','auto','$(FULLEXT)','.packlist').q{ \
-		$(INST_LIB) $(DESTINSTALLPRIVLIB) \
+} unless $self->{NO_PACKLIST};
+
+    push @m,
+q{		$(INST_LIB) $(DESTINSTALLPRIVLIB) \
 		$(INST_ARCHLIB) $(DESTINSTALLARCHLIB) \
 		$(INST_BIN) $(DESTINSTALLBIN) \
 		$(INST_SCRIPT) $(DESTINSTALLSCRIPT) \
@@ -2066,9 +2089,14 @@ pure_perl_install :: all
 
 pure_site_install :: all
 	$(NOECHO) $(MOD_INSTALL) \
-		read }.$self->catfile('$(SITEARCHEXP)','auto','$(FULLEXT)','.packlist').q{ \
+};
+    push @m,
+q{		read }.$self->catfile('$(SITEARCHEXP)','auto','$(FULLEXT)','.packlist').q{ \
 		write }.$self->catfile('$(DESTINSTALLSITEARCH)','auto','$(FULLEXT)','.packlist').q{ \
-		$(INST_LIB) $(DESTINSTALLSITELIB) \
+} unless $self->{NO_PACKLIST};
+
+    push @m,
+q{		$(INST_LIB) $(DESTINSTALLSITELIB) \
 		$(INST_ARCHLIB) $(DESTINSTALLSITEARCH) \
 		$(INST_BIN) $(DESTINSTALLSITEBIN) \
 		$(INST_SCRIPT) $(DESTINSTALLSITESCRIPT) \
@@ -2079,15 +2107,35 @@ pure_site_install :: all
 
 pure_vendor_install :: all
 	$(NOECHO) $(MOD_INSTALL) \
-		read }.$self->catfile('$(VENDORARCHEXP)','auto','$(FULLEXT)','.packlist').q{ \
+};
+    push @m,
+q{		read }.$self->catfile('$(VENDORARCHEXP)','auto','$(FULLEXT)','.packlist').q{ \
 		write }.$self->catfile('$(DESTINSTALLVENDORARCH)','auto','$(FULLEXT)','.packlist').q{ \
-		$(INST_LIB) $(DESTINSTALLVENDORLIB) \
+} unless $self->{NO_PACKLIST};
+
+    push @m,
+q{		$(INST_LIB) $(DESTINSTALLVENDORLIB) \
 		$(INST_ARCHLIB) $(DESTINSTALLVENDORARCH) \
 		$(INST_BIN) $(DESTINSTALLVENDORBIN) \
 		$(INST_SCRIPT) $(DESTINSTALLVENDORSCRIPT) \
 		$(INST_MAN1DIR) $(DESTINSTALLVENDORMAN1DIR) \
 		$(INST_MAN3DIR) $(DESTINSTALLVENDORMAN3DIR)
 
+};
+
+    push @m, q{
+doc_perl_install :: all
+	$(NOECHO) $(NOOP)
+
+doc_site_install :: all
+	$(NOECHO) $(NOOP)
+
+doc_vendor_install :: all
+	$(NOECHO) $(NOOP)
+
+} if $self->{NO_PERLLOCAL};
+
+    push @m, q{
 doc_perl_install :: all
 	$(NOECHO) $(ECHO) Appending installation info to $(DESTINSTALLARCHLIB)/perllocal.pod
 	-$(NOECHO) $(MKPATH) $(DESTINSTALLARCHLIB)
@@ -2121,7 +2169,7 @@ doc_vendor_install :: all
 		EXE_FILES "$(EXE_FILES)" \
 		>> }.$self->catfile('$(DESTINSTALLARCHLIB)','perllocal.pod').q{
 
-};
+} unless $self->{NO_PERLLOCAL};
 
     push @m, q{
 uninstall :: uninstall_from_$(INSTALLDIRS)dirs
@@ -2331,7 +2379,7 @@ $(MAKE_APERL_FILE) : $(FIRST_MAKEFILE) pm_to_blib
         # Skip perl's libraries.
         return if m/^libperl/ or m/^perl\Q$self->{LIB_EXT}\E$/;
 
-	# Skip purified versions of libraries 
+	# Skip purified versions of libraries
         # (e.g., DynaLoader_pure_p1_c0_032.a)
 	return if m/_pure_\w+_\w+_\w+\.\w+$/ and -f "$File::Find::dir/.pure";
 
@@ -2366,7 +2414,7 @@ $(MAKE_APERL_FILE) : $(FIRST_MAKEFILE) pm_to_blib
 	# don't include the installed version of this extension. I
 	# leave this line here, although it is not necessary anymore:
 	# I patched minimod.PL instead, so that Miniperl.pm won't
-	# enclude duplicates
+	# include duplicates
 
 	# Once the patch to minimod.PL is in the distribution, I can
 	# drop it
@@ -2423,7 +2471,7 @@ MAP_PRELIBS   = $Config{perllibs} $Config{cryptlib}
           }
         }
 
-	print STDOUT "Warning: $libperl not found
+	print "Warning: $libperl not found
     If you're going to build a static perl binary, make sure perl is installed
     otherwise ignore this warning\n"
 		unless (-f $lperl || defined($self->{PERL_SRC}));
@@ -2517,8 +2565,12 @@ $(OBJECT) : $(FIRST_MAKEFILE)
 
     my $newer_than_target = $Is{VMS} ? '$(MMS$SOURCE_LIST)' : '$?';
     my $mpl_args = join " ", map qq["$_"], @ARGV;
-
-    $m .= sprintf <<'MAKE_FRAG', $newer_than_target, $mpl_args;
+    my $cross = '';
+    if (defined $::Cross::platform) {
+        # Inherited from win32/buildext.pl
+        $cross = "-MCross=$::Cross::platform ";
+    }
+    $m .= sprintf <<'MAKE_FRAG', $newer_than_target, $cross, $mpl_args;
 # We take a very conservative approach here, but it's worth it.
 # We move Makefile to Makefile.old here to avoid gnu make looping.
 $(FIRST_MAKEFILE) : Makefile.PL $(CONFIGDEP)
@@ -2527,7 +2579,7 @@ $(FIRST_MAKEFILE) : Makefile.PL $(CONFIGDEP)
 	-$(NOECHO) $(RM_F) $(MAKEFILE_OLD)
 	-$(NOECHO) $(MV)   $(FIRST_MAKEFILE) $(MAKEFILE_OLD)
 	- $(MAKE) $(USEMAKEFILE) $(MAKEFILE_OLD) clean $(DEV_NULL)
-	$(PERLRUN) Makefile.PL %s
+	$(PERLRUN) %sMakefile.PL %s
 	$(NOECHO) $(ECHO) "==> Your Makefile has been rebuilt. <=="
 	$(NOECHO) $(ECHO) "==> Please rerun the $(MAKE) command.  <=="
 	$(FALSE)
@@ -2562,7 +2614,7 @@ sub needs_linking {
     my($self) = shift;
 
     my $caller = (caller(0))[3];
-    confess("needs_linking called too early") if 
+    confess("needs_linking called too early") if
       $caller =~ /^ExtUtils::MakeMaker::/;
     return $self->{NEEDS_LINKING} if defined $self->{NEEDS_LINKING};
     if ($self->has_link_code or $self->{MAKEAPERL}){
@@ -2598,9 +2650,15 @@ sub parse_abstract {
         $inpod = /^=(?!cut)/ ? 1 : /^=cut/ ? 0 : $inpod;
         next if !$inpod;
         chop;
-        next unless /^($package(?:\.pm)? \s+ -+ \s+)(.*)/x;
-        $result = $2;
-        last;
+        if ( /^($package(?:\.pm)? \s+ -+ \s+)(.*)/x ) {
+          $result = $2;
+          next;
+        }
+        next unless $result;
+        if ( $result && ( /^\s*$/ || /^\=/ ) ) {
+          last;
+        }
+        $result = join ' ', $result, $_;
     }
     close $fh;
 
@@ -2644,29 +2702,8 @@ sub parse_version {
             local $^W = 0;
             $result = $1;
         }
-        elsif ( m{(?<!\\) ([\$*]) (([\w\:\']*) \bVERSION)\b .* =}x ) {
-            my $eval = qq{
-                package ExtUtils::MakeMaker::_version;
-                no strict;
-                BEGIN { eval {
-                    # Ensure any version() routine which might have leaked
-                    # into this package has been deleted.  Interferes with
-                    # version->import()
-                    undef *version;
-                    require version;
-                    "version"->import;
-                } }
-
-                local $1$2;
-                \$$2=undef;
-                do {
-                    $_
-                };
-                \$$2;
-            };
-            local $^W = 0;
-            $result = eval($eval);  ## no critic
-            warn "Could not eval '$eval' in $parsefile: $@" if $@;
+        elsif ( m{(?<!\\) ([\$*]) (([\w\:\']*) \bVERSION)\b .* (?<![<>=!])\=[^=]}x ) {
+			$result = $self->get_version($parsefile, $1, $2);
         }
         else {
           next;
@@ -2675,10 +2712,44 @@ sub parse_version {
     }
     close $fh;
 
+    if ( defined $result && $result !~ /^v?[\d_\.]+$/ ) {
+      require version;
+      my $normal = eval { version->parse( $result ) };
+      $result = $normal if defined $normal;
+    }
     $result = "undef" unless defined $result;
     return $result;
 }
 
+sub get_version
+{
+	my ($self, $parsefile, $sigil, $name) = @_;
+	my $eval = qq{
+		package ExtUtils::MakeMaker::_version;
+		no strict;
+		BEGIN { eval {
+			# Ensure any version() routine which might have leaked
+			# into this package has been deleted.  Interferes with
+			# version->import()
+			undef *version;
+			require version;
+			"version"->import;
+		} }
+
+		local $sigil$name;
+		\$$name=undef;
+		do {
+			$_
+		};
+		\$$name;
+	};
+  $eval = $1 if $eval =~ m{^(.+)}s;
+	local $^W = 0;
+	my $result = eval($eval);  ## no critic
+	warn "Could not eval '$eval' in $parsefile: $@" if $@;
+	$result;
+}
+
 
 =item pasthru (o)
 
@@ -2697,7 +2768,7 @@ sub pasthru {
 
     foreach my $key (qw(LIB LIBPERL_A LINKTYPE OPTIMIZE
                      PREFIX INSTALL_BASE)
-                 ) 
+                 )
     {
         next unless defined $self->{$key};
 	push @pasthru, "$key=\"\$($key)\"";
@@ -2753,55 +2824,15 @@ MAKE_FRAG
 
     return join "", @m unless $self->needs_linking;
 
-    push @m, q{
-PERL_HDRS = \
-	$(PERL_INC)/EXTERN.h		\
-	$(PERL_INC)/INTERN.h		\
-	$(PERL_INC)/XSUB.h		\
-	$(PERL_INC)/av.h		\
-	$(PERL_INC)/config.h		\
-	$(PERL_INC)/cop.h		\
-	$(PERL_INC)/cv.h		\
-	$(PERL_INC)/dosish.h		\
-	$(PERL_INC)/embed.h		\
-	$(PERL_INC)/embedvar.h		\
-	$(PERL_INC)/fakethr.h		\
-	$(PERL_INC)/form.h		\
-	$(PERL_INC)/gv.h		\
-	$(PERL_INC)/handy.h		\
-	$(PERL_INC)/hv.h		\
-	$(PERL_INC)/intrpvar.h		\
-	$(PERL_INC)/iperlsys.h		\
-	$(PERL_INC)/keywords.h		\
-	$(PERL_INC)/mg.h		\
-	$(PERL_INC)/nostdio.h		\
-	$(PERL_INC)/op.h		\
-	$(PERL_INC)/opcode.h		\
-	$(PERL_INC)/patchlevel.h	\
-	$(PERL_INC)/perl.h		\
-	$(PERL_INC)/perlio.h		\
-	$(PERL_INC)/perlsdio.h		\
-	$(PERL_INC)/perlsfio.h		\
-	$(PERL_INC)/perlvars.h		\
-	$(PERL_INC)/perly.h		\
-	$(PERL_INC)/pp.h		\
-	$(PERL_INC)/pp_proto.h		\
-	$(PERL_INC)/proto.h		\
-	$(PERL_INC)/regcomp.h		\
-	$(PERL_INC)/regexp.h		\
-	$(PERL_INC)/regnodes.h		\
-	$(PERL_INC)/scope.h		\
-	$(PERL_INC)/sv.h		\
-	$(PERL_INC)/thread.h		\
-	$(PERL_INC)/unixish.h		\
-	$(PERL_INC)/util.h
-
-$(OBJECT) : $(PERL_HDRS)
-} if $self->{OBJECT};
+    if ($self->{OBJECT}) {
+        # Need to add an object file dependency on the perl headers.
+        # this is very important for XS modules in perl.git development.
+        push @m, $self->_perl_header_files_fragment("/"); # Directory separator between $(PERL_INC)/header.h
+    }
 
     push @m, join(" ", values %{$self->{XS}})." : \$(XSUBPPDEPS)\n"  if %{$self->{XS}};
 
-    join "\n", @m;
+    return join "\n", @m;
 }
 
 
@@ -2824,7 +2855,8 @@ pm_to_blib : $(FIRST_MAKEFILE) $(TO_INST_PM)
 pm_to_blib({\@ARGV}, '$autodir', q[\$(PM_FILTER)], '\$(PERM_DIR)')
 CODE
 
-    my @cmds = $self->split_command($pm_to_blib, %{$self->{PM}});
+    my @cmds = $self->split_command($pm_to_blib,
+                  map { ($_, $self->{PM}->{$_}) } sort keys %{$self->{PM}});
 
     $r .= join '', map { "\t\$(NOECHO) $_\n" } @cmds;
     $r .= qq{\t\$(NOECHO) \$(TOUCH) pm_to_blib\n};
@@ -2946,7 +2978,7 @@ PPD_OUT
                   $self->{PPM_INSTALL_EXEC}, $self->{PPM_INSTALL_SCRIPT};
         }
         else {
-            $ppd_xml .= sprintf qq{        <INSTALL>%s</INSTALL>\n}, 
+            $ppd_xml .= sprintf qq{        <INSTALL>%s</INSTALL>\n},
                   $self->{PPM_INSTALL_SCRIPT};
         }
     }
@@ -2975,7 +3007,7 @@ PPD_OUT
   $MM->prefixify($var, $prefix, $new_prefix, $default);
 
 Using either $MM->{uc $var} || $Config{lc $var}, it will attempt to
-replace it's $prefix with a $new_prefix.  
+replace it's $prefix with a $new_prefix.
 
 Should the $prefix fail to match I<AND> a PREFIX was given as an
 argument to WriteMakefile() it will set it to the $new_prefix +
@@ -2999,20 +3031,20 @@ simply use '/home/foo/man/man1'.
 sub prefixify {
     my($self,$var,$sprefix,$rprefix,$default) = @_;
 
-    my $path = $self->{uc $var} || 
+    my $path = $self->{uc $var} ||
                $Config_Override{lc $var} || $Config{lc $var} || '';
 
     $rprefix .= '/' if $sprefix =~ m|/$|;
 
-    print STDERR "  prefixify $var => $path\n" if $Verbose >= 2;
-    print STDERR "    from $sprefix to $rprefix\n" if $Verbose >= 2;
+    warn "  prefixify $var => $path\n" if $Verbose >= 2;
+    warn "    from $sprefix to $rprefix\n" if $Verbose >= 2;
 
     if( $self->{ARGS}{PREFIX} &&
-        $path !~ s{^\Q$sprefix\E\b}{$rprefix}s ) 
+        $path !~ s{^\Q$sprefix\E\b}{$rprefix}s )
     {
 
-        print STDERR "    cannot prefix, using default.\n" if $Verbose >= 2;
-        print STDERR "    no default!\n" if !$default && $Verbose >= 2;
+        warn "    cannot prefix, using default.\n" if $Verbose >= 2;
+        warn "    no default!\n" if !$default && $Verbose >= 2;
 
         $path = $self->catdir($rprefix, $default) if $default;
     }
@@ -3142,7 +3174,7 @@ sub oneliner {
 
     $switches = join ' ', @$switches;
 
-    return qq{\$(ABSPERLRUN) $switches -e $cmd --};   
+    return qq{\$(ABSPERLRUN) $switches -e $cmd --};
 }
 
 
@@ -3240,10 +3272,10 @@ END
 	$(CP) $(MYEXTLIB) $@
 MAKE_FRAG
 
-    my $ar; 
+    my $ar;
     if (exists $self->{FULL_AR} && -x $self->{FULL_AR}) {
         # Prefer the absolute pathed ar if available so that PATH
-        # doesn't confuse us.  Perl itself is built with the full_ar.  
+        # doesn't confuse us.  Perl itself is built with the full_ar.
         $ar = 'FULL_AR';
     } else {
         $ar = 'AR';
@@ -3313,7 +3345,7 @@ Helper subroutine for subdirs
 sub subdir_x {
     my($self, $subdir) = @_;
 
-    my $subdir_cmd = $self->cd($subdir, 
+    my $subdir_cmd = $self->cd($subdir,
       '$(MAKE) $(USEMAKEFILE) $(FIRST_MAKEFILE) all $(PASTHRU)'
     );
     return sprintf <<'EOT', $subdir_cmd;
@@ -3364,7 +3396,10 @@ sub test {
 
     my($self, %attribs) = @_;
     my $tests = $attribs{TESTS} || '';
-    if (!$tests && -d 't') {
+    if (!$tests && -d 't' && defined $attribs{RECURSIVE_TEST_FILES}) {
+        $tests = $self->find_tests_recursive;
+    }
+    elsif (!$tests && -d 't') {
         $tests = $self->find_tests;
     }
     # note: 'test.pl' name is also hardcoded in init_dirscan()
@@ -3400,14 +3435,14 @@ END
     push(@m, "\n");
 
     push(@m, "test_dynamic :: pure_all\n");
-    push(@m, $self->test_via_harness('$(FULLPERLRUN)', '$(TEST_FILES)')) 
+    push(@m, $self->test_via_harness('$(FULLPERLRUN)', '$(TEST_FILES)'))
       if $tests;
-    push(@m, $self->test_via_script('$(FULLPERLRUN)', '$(TEST_FILE)')) 
+    push(@m, $self->test_via_script('$(FULLPERLRUN)', '$(TEST_FILE)'))
       if -f "test.pl";
     push(@m, "\n");
 
     push(@m, "testdb_dynamic :: pure_all\n");
-    push(@m, $self->test_via_script('$(FULLPERLRUN) $(TESTDB_SW)', 
+    push(@m, $self->test_via_script('$(FULLPERLRUN) $(TESTDB_SW)',
                                     '$(TEST_FILE)'));
     push(@m, "\n");
 
@@ -3495,10 +3530,10 @@ sub tool_xsubpp {
     }
 
     if ($Is{VMS}                          &&
-        $Config{'ldflags'}               && 
+        $Config{'ldflags'}               &&
         $Config{'ldflags'} =~ m!/Debug!i &&
         (!exists($self->{XSOPT}) || $self->{XSOPT} !~ /linenumbers/)
-       ) 
+       )
     {
         unshift(@tmargs,'-nolinenumbers');
     }
@@ -3513,7 +3548,7 @@ XSUBPPRUN = \$(PERLRUN) \$(XSUBPP)
 XSPROTOARG = $self->{XSPROTOARG}
 XSUBPPDEPS = @tmdeps \$(XSUBPP)
 XSUBPPARGS = @tmargs
-XSUBPP_EXTRA_ARGS = 
+XSUBPP_EXTRA_ARGS =
 };
 };
 
