@@ -4,30 +4,30 @@ use strict ;
 use warnings;
 use bytes;
 
-use IO::Compress::Base::Common  2.060 qw(:Status );
-use IO::Compress::RawDeflate 2.060 ();
-use IO::Compress::Adapter::Deflate 2.060 ;
-use IO::Compress::Adapter::Identity 2.060 ;
-use IO::Compress::Zlib::Extra 2.060 ;
-use IO::Compress::Zip::Constants 2.060 ;
+use IO::Compress::Base::Common  2.063 qw(:Status );
+use IO::Compress::RawDeflate 2.063 ();
+use IO::Compress::Adapter::Deflate 2.063 ;
+use IO::Compress::Adapter::Identity 2.063 ;
+use IO::Compress::Zlib::Extra 2.063 ;
+use IO::Compress::Zip::Constants 2.063 ;
 
 use File::Spec();
 use Config;
 
-use Compress::Raw::Zlib  2.060 (); 
+use Compress::Raw::Zlib  2.063 (); 
 
 BEGIN
 {
     eval { require IO::Compress::Adapter::Bzip2 ; 
-           import  IO::Compress::Adapter::Bzip2 2.060 ; 
+           import  IO::Compress::Adapter::Bzip2 2.063 ; 
            require IO::Compress::Bzip2 ; 
-           import  IO::Compress::Bzip2 2.060 ; 
+           import  IO::Compress::Bzip2 2.063 ; 
          } ;
          
     eval { require IO::Compress::Adapter::Lzma ; 
-           import  IO::Compress::Adapter::Lzma 2.060 ; 
+           import  IO::Compress::Adapter::Lzma 2.063 ; 
            require IO::Compress::Lzma ; 
-           import  IO::Compress::Lzma 2.060 ; 
+           import  IO::Compress::Lzma 2.063 ; 
          } ;
 }
 
@@ -36,7 +36,7 @@ require Exporter ;
 
 our ($VERSION, @ISA, @EXPORT_OK, %EXPORT_TAGS, %DEFLATE_CONSTANTS, $ZipError);
 
-$VERSION = '2.060';
+$VERSION = '2.063';
 $ZipError = '';
 
 @ISA = qw(Exporter IO::Compress::RawDeflate);
@@ -939,7 +939,7 @@ section.
 
 The functional interface needs Perl5.005 or better.
 
-=head2 zip $input => $output [, OPTS]
+=head2 zip $input_filename_or_reference => $output_filename_or_reference [, OPTS]
 
 C<zip> expects at least two parameters,
 C<$input_filename_or_reference> and C<$output_filename_or_reference>.
@@ -1328,7 +1328,7 @@ normalized filename will be passed to the sub.
 If you use C<FilterName> to modify the filename, it is your responsibility
 to keep the filename in Unix format.
 
-Although this option can be used with the OO ointerface, it is of most use
+Although this option can be used with the OO interface, it is of most use
 with the one-shot interface. For example, the code below shows how
 C<FilterName> can be used to remove the path component from a series of
 filenames before they are stored in C<$zipfile>.
@@ -1410,7 +1410,7 @@ and C<$gid>. These values correspond to the numeric User ID (UID) and Group ID
 (GID) of the owner of the files respectively.
 
 When the C<exUnixN> option is present it will trigger the creation of a
-UnixN extra field (ID is "ux") in bothe the local and central zip headers. 
+UnixN extra field (ID is "ux") in both the local and central zip headers. 
 This will be populated with C<$uid> and C<$gid>. 
 The UID & GID are stored as 32-bit integers.
 
@@ -1586,7 +1586,7 @@ Valid values are 0-9 and C<LZMA_PRESET_DEFAULT>.
 0 is the fastest compression with the lowest memory usage and the lowest
 compression.
 
-9 is the slowest compession with the highest memory usage but with the best
+9 is the slowest compression with the highest memory usage but with the best
 compression.
 
 This option is only valid if the C<Method> is ZIP_CM_LZMA. It is ignored
@@ -1833,7 +1833,7 @@ Usage is
 
 Closes the current compressed data stream and starts a new one.
 
-OPTS consists of any of the the options that are available when creating
+OPTS consists of any of the options that are available when creating
 the C<$z> object.
 
 See the L</"Constructor Options"> section for more details.
