@@ -31,8 +31,7 @@ typedef U64TYPE PADOFFSET;
 struct padlist {
     SSize_t	xpadl_max;	/* max index for which array has space */
     PAD **	xpadl_alloc;	/* pointer to beginning of array of AVs */
-    U32		xpadl_id;	/* Semi-unique ID, shared between clones */
-    U32		xpadl_outid;	/* ID of outer pad */
+    PADNAMELIST*xpadl_outid;	/* Padnamelist of outer pad; used as ID */
 };
 
 
@@ -180,7 +179,9 @@ The C array of a padlist, containing the pads.  Only subscript it with
 numbers >= 1, as the 0th entry is not guaranteed to remain usable.
 
 =for apidoc Amx|SSize_t|PadlistMAX|PADLIST padlist
-The index of the last pad in the padlist.
+The index of the last allocated space in the padlist.  Note that the last
+pad may be in an earlier slot.  Any entries following it will be NULL in
+that case.
 
 =for apidoc Amx|PADNAMELIST *|PadlistNAMES|PADLIST padlist
 The names associated with pad entries.
@@ -286,12 +287,14 @@ Restore the old pad saved into the local variable opad by PAD_SAVE_LOCAL()
 
 #define PadnamelistARRAY(pnl)	AvARRAY(pnl)
 #define PadnamelistMAX(pnl)	AvFILLp(pnl)
+#define PadnamelistMAXNAMED(pnl) \
+	((XPVAV*) SvANY(pnl))->xmg_u.xmg_hash_index
 
 #define PadARRAY(pad)		AvARRAY(pad)
 #define PadMAX(pad)		AvFILLp(pad)
 
 #define PadnamePV(pn)		(SvPOKp(pn) ? SvPVX(pn) : NULL)
-#define PadnameLEN(pn)		SvCUR(pn)
+#define PadnameLEN(pn)		((pn) == &PL_sv_undef ? 0 : SvCUR(pn))
 #define PadnameUTF8(pn)		!!SvUTF8(pn)
 #define PadnameSV(pn)		pn
 #define PadnameIsOUR(pn)	!!SvPAD_OUR(pn)
